"""
Telegram API configurations for different platforms.
Replaces opentele.api functionality.
"""


class APIData:
    """Telegram API data class for client configuration."""
    
    def __init__(
        self,
        api_id: int,
        api_hash: str,
        device_model: str,
        system_version: str,
        app_version: str,
        lang_code: str = "en",
        system_lang_code: str = "en-US",
    ):
        self.api_id = api_id
        self.api_hash = api_hash
        self.device_model = device_model
        self.system_version = system_version
        self.app_version = app_version
        self.lang_code = lang_code
        self.system_lang_code = system_lang_code
    
    def copy(self):
        """Create a copy of this APIData instance."""
        return APIData(
            api_id=self.api_id,
            api_hash=self.api_hash,
            device_model=self.device_model,
            system_version=self.system_version,
            app_version=self.app_version,
            lang_code=self.lang_code,
            system_lang_code=self.system_lang_code,
        )


class API:
    """Telegram API configurations for different platforms."""
    
    # Telegram Desktop
    TelegramDesktop = APIData(
        api_id=17349,
        api_hash="344583e45741c457fe1862106095a5eb",
        device_model="Desktop",
        system_version="Windows 10",
        app_version="4.8.0",
        lang_code="en",
        system_lang_code="en-US",
    )
    
    # Telegram Android
    TelegramAndroid = APIData(
        api_id=4,
        api_hash="014b35b6184100b085b0d0572f9b5103",
        device_model="Android",
        system_version="SDK 23",
        app_version="9.7.0",
        lang_code="en",
        system_lang_code="en-US",
    )
    
    # Telegram iOS
    TelegramIOS = APIData(
        api_id=8,
        api_hash="7245de8e747a0d6fbe11f7cc14fcc0bb",
        device_model="iPhone",
        system_version="iOS 15.0",
        app_version="9.7.0",
        lang_code="en",
        system_lang_code="en-US",
    )
    
    # Telegram macOS
    TelegramMacOS = APIData(
        api_id=946,
        api_hash="5f3fb04eac560c6a3d7dd5cacb85e8b0",
        device_model="Mac",
        system_version="macOS 12.0",
        app_version="9.7.0",
        lang_code="en",
        system_lang_code="en-US",
    )




import base64
import secrets
import struct
import time
from typing import Union
from pathlib import Path

import aiosqlite
from pyrogram.client import Client # type: ignore

from ...api import APIData
from ...exceptions import ValidationError

# Схема из новой версии Pyrogram
SCHEMA = """
CREATE TABLE sessions
(
    dc_id          INTEGER PRIMARY KEY,
    server_address TEXT,
    port           INTEGER,
    api_id         INTEGER,
    test_mode      INTEGER,
    auth_key       BLOB,
    date           INTEGER NOT NULL,
    user_id        INTEGER,
    is_bot         INTEGER
);

CREATE TABLE peers
(
    id             INTEGER PRIMARY KEY,
    access_hash    INTEGER,
    type           INTEGER NOT NULL,
    phone_number   TEXT,
    last_update_on INTEGER NOT NULL DEFAULT (CAST(STRFTIME('%s', 'now') AS INTEGER))
);

CREATE TABLE usernames
(
    id       INTEGER,
    username TEXT,
    FOREIGN KEY (id) REFERENCES peers(id)
);

CREATE TABLE update_state
(
    id   INTEGER PRIMARY KEY,
    pts  INTEGER,
    qts  INTEGER,
    date INTEGER,
    seq  INTEGER
);

CREATE TABLE version
(
    number INTEGER PRIMARY KEY
);

CREATE INDEX idx_peers_id ON peers (id);
CREATE INDEX idx_peers_phone_number ON peers (phone_number);
CREATE INDEX idx_usernames_id ON usernames (id);
CREATE INDEX idx_usernames_username ON usernames (username);

CREATE TRIGGER trg_peers_last_update_on
    AFTER UPDATE
    ON peers
BEGIN
    UPDATE peers
    SET last_update_on = CAST(STRFTIME('%s', 'now') AS INTEGER)
    WHERE id = NEW.id;
END;
"""

# IP-адреса центров данных Telegram
TEST = {1: "149.154.175.10", 2: "149.154.167.40", 3: "149.154.175.117"}

PROD = {
    1: "149.154.175.53",
    2: "149.154.167.51",
    3: "149.154.175.100",
    4: "149.154.167.91",
    5: "91.108.56.130",
    203: "91.105.192.100",
}


class PyroSession:
    STRING_FORMAT = ">BI?256sQ?"
    OLD_FORMAT = ">B?256sI?"
    OLD_FORMAT_64 = ">B?256sQ?"
    STRING_SIZE = 351
    STRING_SIZE_64 = 356

    TABLES = {"sessions", "peers", "usernames", "update_state", "version"}

    def __init__(
        self,
        *,
        dc_id: int,
        auth_key: bytes,
        api_id: int | None = None,
        user_id: int | None = None,
        is_bot: bool = False,
        test_mode: bool = False,
        server_address: str | None = None,
        port: int | None = None,
        date: int | None = None,
    ):
        self.dc_id = dc_id
        self.auth_key = auth_key
        self.api_id = api_id
        self.user_id = user_id
        self.is_bot = is_bot
        self.test_mode = test_mode
        self.server_address = server_address or (TEST if test_mode else PROD).get(dc_id)
        self.port = port or (80 if test_mode else 443)
        self.date = date or int(time.time())

    @classmethod
    def from_string(cls, session_string: str):
        """Создать объект PyroSession из session_string."""
        s = session_string + "=" * (-len(session_string) % 4)
        raw_bytes = base64.urlsafe_b64decode(s)

        if len(session_string) in [cls.STRING_SIZE, cls.STRING_SIZE_64]:
            fmt = (
                cls.OLD_FORMAT_64
                if len(session_string) == cls.STRING_SIZE_64
                else cls.OLD_FORMAT
            )
            dc_id, test_mode, auth_key, user_id, is_bot = struct.unpack(fmt, raw_bytes)
            api_id = None
        else:
            dc_id, api_id, test_mode, auth_key, user_id, is_bot = struct.unpack(
                cls.STRING_FORMAT, raw_bytes
            )

        server_address = (TEST if test_mode else PROD).get(dc_id)
        port = 80 if test_mode else 443

        return cls(
            dc_id=dc_id,
            api_id=api_id,
            test_mode=test_mode,
            auth_key=auth_key,
            user_id=user_id,
            is_bot=is_bot,
            server_address=server_address,
            port=port,
        )

    def to_string(self) -> str:
        """Экспортирует сессию в session_string (новый формат)."""
        packed = struct.pack(
            self.STRING_FORMAT,
            self.dc_id,
            self.api_id or 0,
            self.test_mode,
            self.auth_key,
            self.user_id or 0,
            self.is_bot,
        )
        return base64.urlsafe_b64encode(packed).decode().rstrip("=")

    @classmethod
    async def from_file(cls, path: Union[Path, str]):
        """Загружает сессию из .session файла."""
        if not await cls.validate(path):
            raise ValidationError(f"Invalid session structure: {path}")

        async with aiosqlite.connect(path) as db:
            db.row_factory = aiosqlite.Row
            async with db.execute("SELECT * FROM sessions") as cursor:
                session = await cursor.fetchone()

        if session is None:
            raise ValidationError(f"No session found in: {path}")
        return cls(**dict(session))

    @classmethod
    async def validate(cls, path: Union[Path, str]) -> bool:
        """Проверка, что база соответствует новой схеме."""
        try:
            async with aiosqlite.connect(path) as db:
                db.row_factory = aiosqlite.Row
                async with db.execute(
                    "SELECT name FROM sqlite_master WHERE type='table'"
                ) as cur:
                    tables = {row["name"] for row in await cur.fetchall()}

                # должны быть все нужные таблицы
                if not cls.TABLES.issubset(tables):
                    return False

                # минимальная проверка колонок sessions
                async with db.execute("PRAGMA table_info('sessions')") as cur:
                    cols = {r["name"] for r in await cur.fetchall()}
                    expected = {
                        "dc_id",
                        "server_address",
                        "port",
                        "api_id",
                        "test_mode",
                        "auth_key",
                        "date",
                        "user_id",
                        "is_bot",
                    }
                    if not expected.issubset(cols):
                        return False
        except aiosqlite.DatabaseError:
            return False

        return True

    async def to_file(self, path: Union[Path, str]):
        """Создает новый .session файл с текущими данными."""
        async with aiosqlite.connect(path) as db:
            await db.executescript(SCHEMA)
            await db.commit()

            await db.execute(
                "INSERT INTO sessions VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)",
                (
                    self.dc_id,
                    self.server_address,
                    self.port,
                    self.api_id,
                    int(self.test_mode),
                    self.auth_key,
                    self.date,
                    self.user_id,
                    int(self.is_bot),
                ),
            )
            await db.execute("INSERT INTO version VALUES (?)", (7,))
            await db.commit()

    def client(
        self, api: APIData, proxy: dict | None = None, no_updates: bool = True
    ) -> Client:
        """Создает готовый pyrogram.Client на основе этой сессии."""
        return Client(
            name=secrets.token_urlsafe(8),
            api_id=api.api_id,
            api_hash=api.api_hash,
            app_version=api.app_version,
            device_model=api.device_model,
            system_version=api.system_version,
            lang_code=api.lang_code,
            proxy=proxy,
            session_string=self.to_string(),
            no_updates=no_updates,
            test_mode=self.test_mode,
        )



import base64
import secrets
import struct
from typing import Union
from pathlib import Path

import aiosqlite
from pyrogram.client import Client # type: ignore

from ...api import APIData
from ...exceptions import ValidationError


SCHEMA = """
CREATE TABLE sessions (
    dc_id     INTEGER PRIMARY KEY,
    api_id    INTEGER,
    test_mode INTEGER,
    auth_key  BLOB,
    date      INTEGER NOT NULL,
    user_id   INTEGER,
    is_bot    INTEGER
);

CREATE TABLE peers (
    id             INTEGER PRIMARY KEY,
    access_hash    INTEGER,
    type           INTEGER NOT NULL,
    username       TEXT,
    phone_number   TEXT,
    last_update_on INTEGER NOT NULL DEFAULT (CAST(STRFTIME('%s', 'now') AS INTEGER))
);

CREATE TABLE version (
    number INTEGER PRIMARY KEY
);

CREATE INDEX idx_peers_id ON peers (id);
CREATE INDEX idx_peers_username ON peers (username);
CREATE INDEX idx_peers_phone_number ON peers (phone_number);

CREATE TRIGGER trg_peers_last_update_on
    AFTER UPDATE
    ON peers
BEGIN
    UPDATE peers
    SET last_update_on = CAST(STRFTIME('%s', 'now') AS INTEGER)
    WHERE id = NEW.id;
END;
"""


class PyroSession:
    OLD_STRING_FORMAT = ">B?256sI?"
    OLD_STRING_FORMAT_64 = ">B?256sQ?"
    STRING_SIZE = 351
    STRING_SIZE_64 = 356
    STRING_FORMAT = ">BI?256sQ?"
    TABLES = {
        "sessions": {"dc_id", "test_mode", "auth_key", "date", "user_id", "is_bot"},
        "peers": {"id", "access_hash", "type", "username", "phone_number", "last_update_on"},
        "version": {"number"}
    }

    def __init__(
        self,
        *,
        dc_id: int,
        auth_key: bytes,
        user_id: None | int = None,
        is_bot: bool = False,
        test_mode: bool = False,
        api_id: None | int = None,
        date: int | None = None
    ):
        self.dc_id = dc_id
        self.auth_key = auth_key
        self.user_id = user_id
        self.is_bot = is_bot
        self.test_mode = test_mode
        self.api_id = api_id

    @classmethod
    def from_string(cls, session_string: str):
        if len(session_string) in [cls.STRING_SIZE, cls.STRING_SIZE_64]:
            string_format = cls.OLD_STRING_FORMAT_64

            if len(session_string) == cls.STRING_SIZE:
                string_format = cls.OLD_STRING_FORMAT

            api_id = None
            dc_id, test_mode, auth_key, user_id, is_bot = struct.unpack(
                string_format,
                base64.urlsafe_b64decode(
                    session_string + "=" * (-len(session_string) % 4)
                )
            )
        else:
            dc_id, api_id, test_mode, auth_key, user_id, is_bot = struct.unpack(
                cls.STRING_FORMAT,
                base64.urlsafe_b64decode(
                    session_string + "=" * (-len(session_string) % 4)
                )
            )

        return cls(
            dc_id=dc_id,
            api_id=api_id,
            auth_key=auth_key,
            user_id=user_id,
            is_bot=is_bot,
            test_mode=test_mode,
        )

    @classmethod
    async def from_file(cls, path: Union[Path, str]):
        if not await cls.validate(path):
            raise ValidationError()

        async with aiosqlite.connect(path) as db:
            db.row_factory = aiosqlite.Row
            async with db.execute("SELECT * FROM sessions") as cursor:
                session = await cursor.fetchone()

        return cls(**session) # type: ignore

    @classmethod
    async def validate(cls, path: Union[Path, str]) -> bool:
        try:
            async with aiosqlite.connect(path) as db:
                db.row_factory = aiosqlite.Row
                sql = "SELECT name FROM sqlite_master WHERE type='table'"
                async with db.execute(sql) as cursor:
                    tables = {row["name"] for row in await cursor.fetchall()}

                if tables != set(cls.TABLES.keys()):
                    return False

                for table, session_columns in cls.TABLES.items():
                    sql = f'pragma table_info("{table}")'
                    async with db.execute(sql) as cur:
                        columns = {row["name"] for row in await cur.fetchall()}
                        if "api_id" in columns:
                            columns.remove("api_id")
                        if session_columns != columns:
                            return False

        except aiosqlite.DatabaseError:
            return False

        return True

    def client(
        self,
        api: APIData,
        proxy: None | dict = None,
        no_updates: bool = True
    ) -> Client:
        client = Client(
            name=secrets.token_urlsafe(8),
            api_id=api.api_id,
            api_hash=api.api_hash,
            app_version=api.app_version,
            device_model=api.device_model,
            system_version=api.system_version,
            lang_code=api.lang_code,
            proxy=proxy,
            session_string=self.to_string(),
            no_updates=no_updates,
            test_mode=self.test_mode,
        )
        return client

    def to_string(self) -> str:
        packed = struct.pack(
            self.STRING_FORMAT,
            self.dc_id,
            self.api_id or 0,
            self.test_mode,
            self.auth_key,
            self.user_id or 9999,
            self.is_bot
        )
        return base64.urlsafe_b64encode(packed).decode().rstrip("=")

    async def to_file(self, path: Union[Path, str]):
        async with aiosqlite.connect(path) as db:
            await db.executescript(SCHEMA)
            await db.commit()
            sql = "INSERT INTO sessions VALUES (?, ?, ?, ?, ?, ?, ?)"
            params = (
                self.dc_id,
                self.api_id,
                self.test_mode,
                self.auth_key,
                0,
                self.user_id or 9999,
                self.is_bot
            )
            await db.execute(sql, params)
            await db.commit()
            sql = "INSERT INTO version VALUES (?)"
            params = (3,)
            await db.execute(sql, params)
            await db.commit()



from .. import is_kurigram, is_pyrogram

if is_kurigram:
    from .kuri import PyroSession  # noqa: F401
elif is_pyrogram:
    from .pyro import PyroSession  # noqa: F401
else:
    raise ImportError("Must install either kurigram or pyrogram to use PyroSession.")
    
__all__ = ["PyroSession"]



from pathlib import Path
from typing import Union

from .api import API, APIData
try:
    from .sessions.pyro import PyroSession
except ImportError:
    PyroSession = None
try:
    from .sessions.tele import TeleSession
except ImportError:
    TeleSession = None
try:
    from .sessions.tdata import TDataSession
except ImportError:
    TDataSession = None
from .exceptions import ValidationError


class SessionManager:
    def __init__(
        self,
        dc_id: int,
        auth_key: bytes,
        user_id: None | int = None,
        valid: None | bool = None,
        api: APIData = API.TelegramDesktop,
        phone_number: str | None = None,
        test_mode: bool = False,
        is_bot: bool = False,
        api_id: None | int = None,
    ):
        """
        Initializes a SessionManager instance with the specified parameters.

        Args:
            dc_id (int): Data center ID.
            auth_key (bytes): Authentication key.
            user_id (None|int, optional): User ID, default is None.
            valid (None|bool, optional): Validation status, default is None.
            api (APIData, optional): API data, default is API.TelegramDesktop.
            phone_number (str|None, optional): Phone number, default is None.
            test_mode (bool, optional): Test mode flag, default is False.
            is_bot (bool, optional): Is bot flag, default is False.
            api_id (int|None, optional): API ID from session, default is None.
        """

        self.dc_id = dc_id
        self.auth_key = auth_key
        self.user_id = user_id
        self.valid = valid
        self.api = api.copy()
        self.user = None
        self.client = None
        self.phone_number = phone_number
        self.test_mode = test_mode
        self.is_bot = is_bot
        self.api_id = api_id or api.api_id

    async def __aenter__(self):
        """
        Asynchronous context manager entry method.
        Establishes a connection to the Telethon client.
        """
        self.client = self.telethon_client()
        await self.client.connect()
        return self.client

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """
        Asynchronous context manager exit method.
        Disconnects the Telethon client.
        """
        await self.client.disconnect() # type: ignore
        self.client = None

    @property
    def auth_key_hex(self) -> str:
        """
        Returns the hexadecimal representation of the authentication key.
        """
        return self.auth_key.hex()

    @classmethod
    async def from_telethon_file(cls, file: Union[Path, str], api=API.TelegramDesktop):
        """
        Creates a SessionManager instance from a Telethon file.

        Args:
            file (Union[Path, str]): Path to the Telethon file.
            api (APIData, optional): API data, default is API.TelegramDesktop.

        Returns:
            SessionManager: An instance initialized from the Telethon file.
        """
        if TeleSession is None:
            raise ImportError("Must install telethon to use Telethon sessions.")
        session = await TeleSession.from_file(Path(file))
        return cls(
            dc_id=session.dc_id,
            auth_key=session.auth_key,
            api=api,
            phone_number=session.phone_number, # type: ignore
            user_id=session.user_id,
        )

    @classmethod
    def from_telethon_string(cls, string: str, api=API.TelegramDesktop):
        """
        Creates a SessionManager instance from a Telethon string.

        Args:
            string (str): Telethon session string.
            api (APIData, optional): API data, default is API.TelegramDesktop.

        Returns:
            SessionManager: An instance initialized from the Telethon string.
        """
        if TeleSession is None:
            raise ImportError("Must install telethon to use Telethon sessions.")
        session = TeleSession.from_string(string)
        return cls(dc_id=session.dc_id, auth_key=session.auth_key, api=api)

    @classmethod
    async def from_pyrogram_file(cls, file: Union[Path, str], api=API.TelegramDesktop):
        """
        Creates a SessionManager instance from a Pyrogram file.

        Args:
            file (Union[Path, str]): Path to the Pyrogram file.
            api (APIData, optional): API data, default is API.TelegramDesktop.

        Returns:
            SessionManager: An instance initialized from the Pyrogram file.
        """
        if PyroSession is None:
            raise ImportError("Must install pyrogram or kurigram to use Pyrogram sessions.")
        session = await PyroSession.from_file(file)
        return cls(
            auth_key=session.auth_key,
            dc_id=session.dc_id,
            api=api,
            user_id=session.user_id,
            test_mode=session.test_mode,
            is_bot=session.is_bot,
            api_id=session.api_id,
        )

    @classmethod
    def from_pyrogram_string(cls, string: str, api=API.TelegramDesktop):
        """
        Creates a SessionManager instance from a Pyrogram string.

        Args:
            string (str): Pyrogram session string.
            api (APIData, optional): API data, default is API.TelegramDesktop.

        Returns:
            SessionManager: An instance initialized from the Pyrogram string.
        """
        if PyroSession is None:
            raise ImportError("Must install pyrogram or kurigram to use Pyrogram sessions.")
        session = PyroSession.from_string(string)
        return cls(
            auth_key=session.auth_key,
            dc_id=session.dc_id,
            api=api,
            user_id=session.user_id,
            test_mode=session.test_mode,
            is_bot=session.is_bot,
            api_id=session.api_id,
        )

    @classmethod
    def from_tdata_folder(cls, folder: Union[Path, str]):
        """
        Creates a SessionManager instance from a TData session folder.

        Args:
            folder (Union[Path, str]): Path to the TData session folder.

        Returns:
            SessionManager: An instance initialized from the TData session folder.
        """
        if not TDataSession:
            raise ImportError(
                "TData support requires opentele package. "
                "Please install it with: pip install tgconvertor[tdata] or pip install opentele"
            )
        session = TDataSession.from_tdata(folder)
        return cls(auth_key=session.auth_key, dc_id=session.dc_id, api=session.api)

    async def to_pyrogram_file(self, path: Union[Path, str]):
        """
        Saves the current session as a Pyrogram file.

        Args:
            path (Union[Path, str]): Path to save the Pyrogram file.
        """
        await self.pyrogram.to_file(Path(path))

    def to_pyrogram_string(self) -> str:
        """
        Converts the current session to a Pyrogram session string.

        Returns:
            str: Pyrogram session string.
        """
        return self.pyrogram.to_string()

    async def to_telethon_file(self, path: Union[Path, str]):
        """
        Saves the current session as a Telethon file.

        Args:
            path (Union[Path, str]): Path to save the Telethon file.
        """
        await self.telethon.to_file(Path(path))

    def to_telethon_string(self) -> str:
        """
        Converts the current session to a Telethon session string.

        Returns:
            str: Telethon session string.
        """
        return self.telethon.to_string()

    async def to_tdata_folder(self, path: Union[Path, str]):
        """
        Saves the current session as a TData session folder.

        Args:
            path (Union[Path, str]): Path to save the TData session folder.
        """
        await self.get_user_id()
        self.tdata.to_folder(path)

    @property
    def pyrogram(self) -> PyroSession:
        """
        Returns a PyroSession instance representing the current session.
        """

        if PyroSession is None:
            raise ImportError("Must install pyrogram or kurigram to use Pyrogram sessions.")

        return PyroSession(
            dc_id=self.dc_id,
            auth_key=self.auth_key,
            user_id=self.user_id,
            api_id=self.api_id,
            test_mode=self.test_mode,
            is_bot=self.is_bot,
        )

    @property
    def telethon(self) -> TeleSession:
        """
        Returns a TeleSession instance representing the current session.
        """
        if TeleSession is None:
            raise ImportError("Must install telethon to use Telethon sessions.")
        return TeleSession(
            dc_id=self.dc_id,
            auth_key=self.auth_key,
        )

    @property
    def tdata(self) -> TDataSession:
        """
        Returns a TDataSession instance representing the current session.
        """
        if not TDataSession:
            raise ImportError(
                "TData support requires opentele package. "
                "Please install it with: pip install tgconvertor[tdata] or pip install opentele"
            )
        if self.user_id is None:
            raise ValueError("user_id is required for TDataSession")
        return TDataSession(
            dc_id=self.dc_id,
            auth_key=self.auth_key,
            api=self.api,
            user_id=self.user_id,
        )

    def pyrogram_client(self, proxy=None, no_updates=True):
        """
        Returns a Pyrogram client for the current session.

        Args:
            proxy: Proxy information for the client.
            no_updates (bool): Flag indicating whether to disable updates.

        Returns:
            Pyrogram client instance.
        """
        client = self.pyrogram.client(
            api=self.api,
            proxy=proxy,
            no_updates=no_updates,
        )
        return client

    def telethon_client(self, proxy=None, no_updates=True):
        """
        Returns a Telethon client for the current session.

        Args:
            proxy: Proxy information for the client.
            no_updates (bool): Flag indicating whether to disable updates.

        Returns:
            Telethon client instance.
        """
        client = self.telethon.client(
            api=self.api,
            proxy=proxy,
            no_updates=no_updates,
        )
        return client

    async def validate(self) -> bool:
        """
        Validates the current session.

        Returns:
            bool: Validation status (True if valid, False otherwise).
        """
        user = await self.get_user()
        self.valid = bool(user)
        return self.valid

    async def get_user_id(self) -> int:
        """
        Gets the user ID associated with the current session.

        Returns:
            int: User ID.

        Raises:
            ValidationError: If the user is not available.
        """
        if self.user_id:
            return self.user_id

        user = await self.get_user()

        if user is None:
            raise ValidationError()

        return user.id # type: ignore

    async def get_user(self):
        """
        Gets the user information associated with the current session.

        Returns:
            User: User information.

        Raises:
            ValidationError: If the user is not available.
        """
        async with self as client:
            self.user = await client.get_me()
            if self.user:
                self.user_id = self.user.id # type: ignore
        return self.user



from importlib.metadata import distribution, PackageNotFoundError

try:
    distribution("kurigram")
    is_kurigram = True
except PackageNotFoundError:
    is_kurigram = False

try:
    distribution("pyrogram")
    is_pyrogram = True
except PackageNotFoundError:
    is_pyrogram = False

if is_kurigram and is_pyrogram:
    raise ImportError(
        "Both 'kurigram' and 'pyrogram' packages are installed. "
        "Please uninstall one of them to avoid conflicts."
    )